//-----------------------------------------------------------
//  the basic Button Class.
//  Buttons are Instances of HUDOverlay so they can use their own Renderfunction.
//  Provides Button Graphics and basic MouseOver, LeftBtn and RightBtn functions.
//  checks if Mouse is Over and sets Mouse.selectedButton once LeftBtn is clicked
//
//  Copyright 2005 Friedrich Kirschner
//-----------------------------------------------------------
class MSBGuiOverlay extends HUDOverlay
config (MSBButtonConfig);

var float xPos,               //position and area on screen - xPosRel is for resolution independent Buttons
          yPos,
          xPosRel,
          yPosRel,
          width,
          height,
          widthRel,
          heightRel;

var int u,v;                          // Icon Resolution

var config color col,                 //Button Color,Text Color, Tooltip color,MouseOver color
                 textCol,
                 tooltipCol,
                 overlayCol;

var MSBMouseOverlay Mouse;            //Pointer to Mouse
var MSBController M;                  //Pointer to PlayerController

var bool bOver;
var bool bIsSelected;                 //is this button selected?
var bool hasClicked;                  // used for Two-way Buttons in Cinjunction with bHasClickedAgain
var bool rHasClicked;
var() int SubLevel;                   // indicates the Buttons Menu "deepness".
var bool bEnabled;
var bool bMainMenu;
var() bool bPermanent;                // Left on screen when playing
var bool bContained;                  //won't draw when contained
var bool bDragable;
var bool bDragged;

var config Material Mat;              //Button Icon
var string tooltip;                   //Button Tooltip
var string BtnName;

var string FontNameSmall, FontNameMedium, FontNameBig;

var config float tooltipFontScale;
var config float buttonFontScale;
var config float fontScale;

event PostBeginPlay()                //establish Pointer to PlayerController and define Screenarea
{
super.PostBeginPlay();
M=MSBController(Level.GetLocalPlayerController());
if (Mat!=None)
    {
    u=Mat.MaterialUSize();
    v=Mat.MaterialVSize();
    }

//push Button below Mouse
if (M.myHUD!=None)
  sortButtonList();

relativeWidthHeight();
}

function updateMaterial()
{
if (Mat!=None)
    {
    u=Mat.MaterialUSize();
    v=Mat.MaterialVSize();
    }
}

function sortButtonList()
{
local Hud h;
local MSBGuiOverlay mOverlay;

h=M.myHUD;
//first pop the mouse
mOverlay=MSBGuiOverlay(h.Overlays[h.overlays.length-1]);
h.RemoveHudOverlay(mOverlay);
//connect the Mouse
Mouse=MSBMouseOverlay(mOverlay);
//then add ouselves
h.AddHudOverlay(self);
// and PUSH the mouse again
h.AddHudOverlay(mOverlay);
}

function relativeWidthHeight()
{
if (heightRel>0 && M.ScreenY>0)
  height=(heightRel/768)*M.ScreenY;

if (widthRel>0 && M.ScreenX>0)
  width=(widthRel/1024)*M.ScreenX;

if (xPosRel>0 && M.ScreenX>0)
  xPos=(xPosRel/1024)*M.ScreenX;

if (yPosRel>0 && M.ScreenY>0)
  yPos=(yPosRel/768)*M.ScreenX;
}

function Render(Canvas C)           //display Button and call MouseOver, LeftBtn, RightBtn
{
relativeWidthHeight();
if (!bDragable)
  {
  if (xPosRel>0)
    {
    xPos=xPosRel/1024*C.ClipX;
    if (xPos>C.ClipX)
      xPos=C.ClipX-100;
    }
  if (yPosRel>0)
    {
    yPos=yPosRel/768*C.ClipY;
    if (yPos>C.ClipY)
      yPos=C.ClipY-100;
    }
  }


if (M.screenX>=1280)
  C.Font=Font(DynamicLoadObject(FontNameMedium, class'Font'));
else
  C.Font=Font(DynamicLoadObject(FontNameSmall, class'Font'));
C.FontScaleX=1;
C.FontScaleY=1;
initButton(C);
bOver=checkover(Mouse.xpos,Mouse.ypos);
bIsSelected=checkSelected();
if (bOver)
  {
  Mouse.hudTargetActor=self;     //pointer from Mouse to Button it hovers on
  MouseOver(C);
  }

if (bIsSelected)
  {
  IsSelected(C);
  }

if (bEnabled)
  {
  if (Mouse.btnOne && !hasclicked && bOver)
     {
      hasClicked=true;
      LeftBtn(C);
     }
  if (Mouse.btnTwo && !rHasclicked && bOver)
    {
     rHasClicked=true;
     RightBtn(C);
    }
  }
}

//Multi selecting Buttons will work as follows:
//we can "Paint" Buttons On or Off when pressing the shift key and moving over them.
//the shift key can also be held down to instantly paint buttons

function bool checkOver(float X, float Y)  //check if Mouse is over Button
{
if (x>xPos && x<xPos+width)
  if (y>yPos && y<yPos+height)
    return true;
else
  return false;
}

function bool checkSelected()
{
//if the Mouse is over the button and shift is pressed, we are being selected
if (bOver && Mouse.bShiftDown)
  return true;
if (bOver && Mouse.bShiftRelease)
  return false;
if (bIsSelected)
  return true;
}

function initButton(Canvas C)             //draw the Button on Screen
{

C.SetDrawColor(col.R,col.G, col.B, col.A);
C.SetPos(xPos,yPos);
if (Mat!=None)
    C.DrawTile(Mat,width,height,0,0,U-1,V-1);
C.SetPos(xPos,yPos);
}

function LeftBtn(Canvas C)
{
local int i;
// destroy other subMenus at same level
for (i=0;i<M.myHUD.Overlays.length;i++)
  {
  if (M.myHUD.Overlays[i]!=self)  //don't deselect self!!!
    MSBGuiOverlay(M.myHUD.Overlays[i]).deselect();
  if (MSBGuiOverlay(M.myHUD.Overlays[i]).SubLevel>Sublevel)
    M.myHUD.Overlays[i--].Destroy();
  }
Mouse.btnOne=false;
}

function RightBtn(Canvas C)
{
Mouse.btnTwo=false;
}

function MouseOver(Canvas C)
{
C.SetDrawColor(overlayCol.R,overlayCol.G, overlayCol.B, overlayCol.A);
C.SetPos(xPos,yPos);
if (Mat!=None)
  C.DrawTile(Mat,width,height,0,0,U,V);
C.SetPos(xPos+width,yPos+12);
C.SetDrawColor(tooltipCol.R,tooltipCol.G, tooltipCol.B, tooltipCol.A);
C.DrawText(tooltip);
}

function IsSelected(Canvas C);

function deSelect()
{
local int i;
hasClicked=false;
rHasClicked=false;
bEnabled=true;
bIsSelected=false;

//remove Button from selection List
for (i=0;i<Mouse.selectedButtons.Length;i++)
  if (Mouse.selectedButtons[i]==self)
    {
    Mouse.selectedButtons[i]=None;
    Mouse.selectedButtons.Remove(i,1);
    }
}

function deSelectSubLevel()
{
local int i;
for (i=0;i<M.myHUD.Overlays.length;i++)
  {
  MSBGuiOverlay(M.myHUD.Overlays[i]).deSelect();
  if (MSBGuiOverlay(M.myHUD.Overlays[i]).SubLevel>=Sublevel)
    M.myHUD.Overlays[i--].Destroy();
  }
}

defaultproperties
{
FontNameSmall="MovieSandboxTextures.Font.MSBSmallFont"
FontNameMedium="MovieSandboxTextures.Font.MSBSmallFont"
FontNameBig="MovieSandboxTextures.Font.MSBLargeFont"
bEnabled=true
tooltip="Your tip here!"
Mat=Texture'MoviesandboxTextures.icon_node'
xposRel=0
yposRel=0
widthRel=30
heightRel=30
col=(R=255,G=255,B=255,A=255)
TextCol=(R=255,G=255,B=255,A=255)
tooltipCol=(R=255,G=255,B=255,A=255)
overlayCol=(R=255,G=0,B=0,A=100)

fontScale=1
buttonFontScale=1
tooltipFontScale=1
}
