class MSBSpritePawn extends MSBActor
config (MSBSpritePawn) placeable;

var bool bEdit;       //for testing Movement Animations
var bool disablePhysics;  //for having the pawn fly

// movement related
var () int mySphere;
var () int steerAmp;
var vector nextMove;
var () float Speed;
var () float RotateSpeed;
var float steps;     // for movement quantisation
var float moveSteps; // when to move;

// display related
var () Material IdleAni[4];
var () Material MoveAni[4];
var () Material ScriptAni[4];
var Material View[4];
var vector proportion;
var float xSize,ySize;
var string ViewAppendix[12];

//animation related;
var float AniLength;


struct SpriteAnim
{
var string Angle[4];
};

var config array<SpriteAnim> Aniname;

var Actor MoveTarget;
var PlayerController ViewPC;

//***************************************************************************
// general setup and viewadjustments
//***************************************************************************

event PostBeginPlay()
{
super.PostBeginPlay();
ViewPC=Level.GetLocalPlayerController();
SetTimer(0.1,true);                     //for view switching
if (IdleAni[0]!=None)
  texture=IdleAni[0];
}

function LoadProp()
{
local MSB2DCharacterSave char;
local int ani,angle;
Log("Loading:"@content);
char=Level.Game.LoadDataObject(class'MSB2DCharacterSave',content,"MSB2DChar_"$content);

for(angle=0;angle<4;angle++)
  IdleAni[angle]=Material(DynamicLoadObject(char.Idle$ViewAppendix[angle],class'Material'));

if (char.Move!="")
  for(angle=0;angle<4;angle++)
    MoveAni[angle]=Material(DynamicLoadObject(char.Move$ViewAppendix[angle+4],class'Material'));

for(ani=0;ani<char.animations.length;ani++)
  {
  AniName.Insert(AniName.Length,1);
  for(angle=0;angle<4;angle++)
    {
    Aniname[ani].Angle[angle]=char.Animations[ani]$ViewAppendix[angle+8];
    Log(char.Animations[ani]);
    }
  }

SetCollisionSize(16*1,16*2);
if (disablePhysics)
  SetPhysics(PHYS_None);
}

//********************************************************************************
// MSBSpritePawn Algorithms

simulated function Tick(float DeltaTime)
{
super.Tick(DeltaTime);
}

simulated function Timer()
{
if (ViewPC!=none && DrawType==DT_Sprite)
   AdjustSkin();
}


simulated function AdjustSkin()
{
local vector a, b,c;
local vector Origin, Target;
local float alpha;

moveSteps=1/Texture(texture).MaxFrameRate;
if (moveSteps==0)
  moveSteps=12;

//eliminate Z-Component of Vector comparison
Origin=ViewPC.ViewTarget.Location;
Origin.Z=0;
Target=Location;
Target.Z=0;

a=Normal(Origin-Target);
b=Normal(vector(Rotation));
c=a cross b;
alpha= a dot b;

if ( (alpha<=0.7) && (alpha>=-0.7))// Left/Right
  {
  if(c.Z>0)
   {
   if (View[0]!=None)
    Texture=View[0];  //Left
   }
  else
   {
   if (View[1]!=None)
    Texture=View[1];  //Right
   }
  }
if (alpha<-0.7)                     //Back/Front
   {
   if (View[2]!=None)
    Texture=View[2];  //Back
   }
if(alpha>0.7)
   {
   if (View[3]!=None)
    Texture=View[3];  //Front
   }
}

//***************************************************************************
// "global" state specific functions
//***************************************************************************
simulated function bool changeAnimation(name AniSet)
{
local int i;

 switch(AniSet)
 {
  case 'Idle':                                 // switch to idle Set
    for (i=0;i<4;i++)
     {
     if (IdleAni[i]!=None)
       View[i]=IdleAni[i];
     }
    return true;

  case 'Script':                                 // switch to Script Set
    for (i=0;i<4;i++)
     {
     if (ScriptAni[i]!=None)
      View[i]=ScriptAni[i];
     }
    return true;

  case 'Move':                                 // switch to idle Set
    for (i=0;i<4;i++)
     {
     if (MoveAni[i]!=None)
     View[i]=MoveAni[i];
     }
    return true;
 }
}

//***************************************************************************
// state moving
//***************************************************************************

state SpriteMoving
{

 simulated function Tick(float DeltaTime)
 {
  local vector myMove;

  global.Tick(DeltaTime);
  SetPhysics(PHYS_None);
  steps+=DeltaTime;
  if (!bEdit)
    {
    if (moveTarget!=None)
     {
     myMove=AdjustMoveDir();               //evasion
     AdjustPosition(myMove, DeltaTime);    //actual movement
     }
    else gotoState('Idle');                //target reached -> OUT to Idle!!
    }
 }

 function vector AdjustMoveDir()
 {
  local vector myMove,myMoveN,link;
  local float d,proj,turn;
  local Actor A;

  myMove=moveTarget.Location-Location;

  foreach VisibleCollidingActors (class'Actor', A, mySphere)//search actors around us
   {
    if (A!=self && A!=MoveTarget)
     {
     link=Location-A.Location;
     d=(mySphere-VSize(link))/mySphere;
     myMoveN=Normal(myMove cross vect(0, 0, 1));    //generate a Normal to our heading
     proj=(myMoveN dot Normal(link));
     proj+=0.01;
     turn=d/proj*steerAmp;      //project onto normal to get steering
     myMove=myMove+myMoveN*turn;                         //convert local steering to global coordinate vector
    }
    if (VSize(Location-moveTarget.Location)<mySphere)
     {
     moveTarget=None;
     Log("reached my Target"@name);
     return (myMove*0);
    }
   }
  return myMove;
 }


 function AdjustPosition(vector myMove, float DeltaTime)
 {
  myMove=Normal(myMove)*Speed*DeltaTime;
  nextMove+=myMove;
  if (steps>=moveSteps && DrawType==DT_Sprite)
  {
   steps=0;
   nextMove.Z=0;
   SetLocation(Location+nextMove);
   nextMove=nextMove*0;
   if (!disablePhysics)
     SetPhysics(PHYS_Walking);
  }
  if (DrawType!=DT_Sprite)
   {
   steps=0;
   SetLocation(Location+nextMove);
   nextMove=nextMove*0;
   SetPhysics(PHYS_Flying);
  }

  if (moveTarget!=None && VSize(moveTarget.Location-Location)>100)
   SetRotation (rotator(moveTarget.Location-Location));
 }
Begin:
changeAnimation('Move');
if (MoveAni[3]!=None)
  {
  xSize=MoveAni[3].MaterialUSize();
  ySize=MoveAni[3].MaterialVSize();
  proportion.X=xSize/ySize;
  SetDrawScale3D(proportion);
  }
}


state SpriteRotating
{

 simulated function Tick(float DeltaTime)
 {
   global.Tick(DeltaTime);
 }

Begin:
changeAnimation('Move');
}


//***************************************************************************
// state idle
//***************************************************************************
auto State Idle
{
 simulated function Tick(float DeltaTime)
 {
  global.Tick(DeltaTime);
  if (!bEdit)
    {
    if (ScriptAni[3]!=None)
      gotoState('Animating');     //OUT -> state anim
    if (MoveTarget!=None)
      gotoState('SpriteMoving');         //OUT -> state moving
    }
 }

Begin:
changeAnimation('Idle');
if (IdleAni[3]!=None)
  {
  xSize=IdleAni[3].MaterialUSize();
  ySize=IdleAni[3].MaterialVSize();
  proportion.X=xSize/ySize;
  Log("from SpritePawn:Proportion:"@proportion);
  SetDrawScale3D(proportion);
  }
}

//***************************************************************************
// state animating
//***************************************************************************
State Animating
{
Begin:
changeAnimation('Script');
if (ScriptAni[3]!=None)
  {
  xSize=ScriptAni[3].MaterialUSize();
  ySize=ScriptAni[3].MaterialVSize();
  proportion.X=xSize/ySize;
  SetDrawScale3D(proportion);
  }
}


event Trigger(Actor Other, Pawn EventInstigator)
{
if (Other.IsA('MSBPropertyButtonBool'))
  {
  if (disablePhysics)
    {
    SetPhysics(PHYS_None);
    oldPhysics=(PHYS_None);
    }
  else
    {
    SetPhysics(PHYS_Walking);
    oldPhysics=(PHYS_Walking);
    }
  }
}

defaultproperties
{
mySphere=32;
Speed=100;
RotateSpeed;
steerAmp=2;
Mass=0.1;
DrawScale=0.15;

proportion=(X=1,Y=1,Z=1)

bBounce=false;
DrawType=DT_Sprite

bAlwaysRelevant=true;
Physics=PHYS_None;
bCollideWhenPlacing=true;
disablePhysics=true;

bStatic=false
bNoDelete=false
bHidden=false

bDirectional=True

CollisionRadius=16.0
CollisionHeight=16.0

bUseCylinderCollision=true
bCollideActors=True
bBlockActors=False
bCollideWorld=True
bProjTarget=True
Texture=S_Actor

ViewAppendix[0]="IdleL_A00";
ViewAppendix[1]="IdleR_A00";
ViewAppendix[2]="IdleB_A00";
ViewAppendix[3]="IdleF_A00";
ViewAppendix[4]="MoveL_A00";
ViewAppendix[5]="MoveR_A00";
ViewAppendix[6]="MoveB_A00";
ViewAppendix[7]="MoveF_A00";
ViewAppendix[8]="AnimL_A00";
ViewAppendix[9]="AnimR_A00";
ViewAppendix[10]="AnimB_A00";
ViewAppendix[11]="AnimF_A00";

ButtonList(5)="MSBPropertyButtons.MSBPropertyButtonBool"
Properties(5)="disablePhysics"
}

